>From 094d81c8be0b09e77ece728b888e24f4a1b4694a Mon Sep 17 00:00:00 2001
From: Joseph Tulou <brezhoneg1@yahoo.fr>
Date: Fri, 13 Feb 2009 13:36:26 +0100
Subject: [PATCH] corrects two issues related to autoscale/scalefactor
   - On windows, prevents scaling to exceed the container
   - original width/height assumption matches other part of vlc
---
 src/video_output/vout_pictures.c |   37 ++++++++++++++++++++++++++-----------
 1 files changed, 26 insertions(+), 11 deletions(-)

diff --git a/src/video_output/vout_pictures.c b/src/video_output/vout_pictures.c
index 3275aac..abe696c 100644
--- a/src/video_output/vout_pictures.c
+++ b/src/video_output/vout_pictures.c
@@ -465,25 +465,40 @@ void vout_PlacePicture( const vout_thread_t *p_vout,
     int i_zoom = p_vout->i_zoom;
 
     /* be realistic, scaling factor confined between .2 and 10. */
-    if( i_zoom > 10 * ZOOM_FP_FACTOR  || i_zoom <  ZOOM_FP_FACTOR / 5 )
-        i_zoom = ZOOM_FP_FACTOR;
+    if( i_zoom > 10 * ZOOM_FP_FACTOR )      i_zoom = 10 * ZOOM_FP_FACTOR;
+    else if( i_zoom <  ZOOM_FP_FACTOR / 5 ) i_zoom = ZOOM_FP_FACTOR / 5;
 
     if( b_autoscale )
     {
         *pi_width = i_width;
         *pi_height = i_height;
     }
-    else if( i_zoom == ZOOM_FP_FACTOR )  /* original size */
-    {
-        *pi_width = __MIN( i_width, p_vout->fmt_in.i_visible_width );
-        *pi_height = __MIN( i_height, p_vout->fmt_in.i_visible_height );
-    }
     else 
     {
-        *pi_width = 
-             p_vout->fmt_in.i_visible_width * i_zoom / ZOOM_FP_FACTOR;
-        *pi_height =
-             p_vout->fmt_in.i_visible_height * i_zoom / ZOOM_FP_FACTOR;
+        unsigned int i_original_width, i_original_height;
+
+        if( p_vout->fmt_in.i_sar_num > p_vout->fmt_in.i_sar_den )
+        {
+            i_original_width = p_vout->fmt_in.i_visible_width *
+                               p_vout->fmt_in.i_sar_num / p_vout->fmt_in.i_sar_den;
+            i_original_height =  p_vout->fmt_in.i_visible_height;
+        }
+        else
+        {
+            i_original_width =  p_vout->fmt_in.i_visible_width;
+            i_original_height = p_vout->fmt_in.i_visible_height *
+                                p_vout->fmt_in.i_sar_den / p_vout->fmt_in.i_sar_num;
+        }
+       
+#ifdef WIN32
+        /* On windows, inner video window exceeding container leads to black screen */
+        *pi_width = __MIN( i_width, i_original_width * i_zoom / ZOOM_FP_FACTOR );
+        *pi_height = __MIN( i_height, i_original_height * i_zoom / ZOOM_FP_FACTOR );
+#else
+        *pi_width = i_original_width * i_zoom / ZOOM_FP_FACTOR ;
+        *pi_height = i_original_height * i_zoom / ZOOM_FP_FACTOR ;
+#endif
+
     }
 
      int64_t i_scaled_width = p_vout->fmt_in.i_visible_width * (int64_t)p_vout->fmt_in.i_sar_num *
-- 
1.5.2.5

